// InterruptTest.cpp
//
// Applied Data Systems
//
// This sample application shows the use of the 
// ADSInterrupt class object.  ADSInterrupt enables
// easy capturing of external hardware interrupts with
// ADS systems.
//
// History
// -------
// rev 1: November 4, 2002 (ctacke)
//

#include "ADSInterrupt.h"

#define MY_EVENT_NAME  (_T("MyEvent"))
#define CONNECT_EVENT_NAME  (_T("Connected"))

DWORD WINAPI InterruptThread(LPVOID p);

HANDLE hMyEvent = NULL;
HANDLE hConnected = NULL;
ADSInterrupt ADSInt;

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	DWORD dThreadID = 0;

	RETAILMSG (TRUE, (_T("\r\n---- Starting test  ----\r\n")));
	
	// create an event to wait on
	hMyEvent = CreateEvent(NULL, FALSE, FALSE, MY_EVENT_NAME);
	hConnected = CreateEvent(NULL, FALSE, FALSE, CONNECT_EVENT_NAME);

	// start a thread to handle the interrupt events (IST)
	CreateThread(NULL,
				 0,
				 InterruptThread,
    	    	 NULL,
    	    	 0,
    	    	 &dThreadID
    	    	 );

	// make sure the interrupt connected
	DWORD dret = 0, count = 0;
	do
	{
		dret = WaitForSingleObject(hConnected, 200);

		if (dret == WAIT_OBJECT_0)
			break;

		if (count++ > 10) // 2 seconds
		{
			RETAILMSG (TRUE, (_T("\r\n---- Test complete  ----\r\n")));
			return 0;
		}

	} while(dret = WAIT_TIMEOUT);

	// run for five minutes, max
	for (int min_left = 5 ; min_left > 0 ; min_left--)
	{
		RETAILMSG (TRUE, (_T("\r\n---- %d minutes left in test ----\r\n\n"),min_left));
		Sleep(60*1000);
	}

	// disable the interrupt
	ADSInt.Disable();

	// disconnect our event from the interrupt
	ADSInt.Disconnect();

	RETAILMSG (TRUE, (_T("\r\n---- Test complete  ----\r\n")));
	return 0;
}

DWORD WINAPI InterruptThread(LPVOID p)
{
	// this is our IST
	DWORD dret = 0;

	// connect the Bitsy EXT_IRQ1, rising edge
	ADSInt.Connect(BITSY_PLUS_EXT_IRQ1, INTR_MODE_EDGE | INTR_MODE_NEGATIVE_LOGIC);

	// initialize it to fire my event
	if(ADSInt.Initialize(hMyEvent) != 0)
	{
		RETAILMSG (TRUE, (_T(" FAILED: Cannot initialize interrupt\r\n")));
		return 0;
	}

	// let the app thread know we've cconnected
	SetEvent(hConnected);

	while(true)
	{
		// wait on our custom event to be fired by the interrupt
		dret = WaitForSingleObject(hMyEvent, 5000);

		if (dret == WAIT_OBJECT_0)
		{
			// let the kernel know we're done handling the interrupt
			// if we don't it won't get re-enabled
			ADSInt.Done();

			// we've gotten an interrupt
			RETAILMSG (TRUE, (_T(" Interrupt triggered\r\n")));
		}
		else if(dret == WAIT_ABANDONED)
		{
			RETAILMSG (TRUE, (_T(" WAIT_ABANDONED\r\n")));
		}
		else if (dret==WAIT_TIMEOUT)
		{
			// our wait timed out
			RETAILMSG (TRUE, (_T(" WAIT_TIMEOUT\r\n")));
		}
	}
}